export declare function createStyle(doc: Document, cssText: string): HTMLStyleElement;
export declare function createIFrame(parent?: HTMLElement): HTMLIFrameElement;
export interface PrintdCallbackArgs {
    /** Iframe reference */
    iframe: HTMLIFrameElement;
    /** HTMLElement copy reference */
    element?: HTMLElement;
    /** Function to launch the print dialog after content was loaded */
    launchPrint: Function;
}
export declare type PrintdCallback = (args: PrintdCallbackArgs) => void;
/** Printd class that prints HTML elements in a blank document */
export default class Printd {
    private readonly parent;
    private readonly iframe;
    private loading;
    private callback?;
    private elCopy?;
    constructor(parent?: HTMLElement);
    /** Gets current Iframe reference */
    getIFrame(): HTMLIFrameElement;
    /**
     * Print an HTMLElement
     *
     * @param el HTMLElement
     * @param cssText Optional CSS Text that will add to head section of the iframe document
     * @param callback Optional callback that will be triggered when content is ready to print
     */
    print(el: HTMLElement, cssText?: string, callback?: PrintdCallback): void;
    /**
     * Print an URL
     *
     * @param url URL to print
     * @param callback Optional callback that will be triggered when content is ready to print
     */
    printURL(url: string, callback?: PrintdCallback): void;
    private launchPrint;
    private loadEvent;
}
export { Printd };
